@@ -4,7 +4,9 @@ package DBD::SQLite::VirtualTable::PerlData;
 use strict;
 use warnings;
 use base 'DBD::SQLite::VirtualTable';
-
+use DBD::SQLite;
+use constant SQLITE_3010000 => $DBD::SQLite::sqlite_version_number >= 3010000 ? 1 : 0;
+use constant SQLITE_3021000 => $DBD::SQLite::sqlite_version_number >= 3021000 ? 1 : 0;
 
 # private data for translating comparison operators from Sqlite to Perl
 my $TXT = 0;
@@ -17,6 +19,18 @@ my %SQLOP2PERLOP = (
   '>'     => [ 'gt',   '>'  ],
   '>='    => [ 'ge',   '>=' ],
   'MATCH' => [ '=~',   '=~' ],
+  (SQLITE_3010000 ? (
+  'LIKE'  => [ 'DBD::SQLite::strlike', 'DBD::SQLite::strlike' ],
+  'GLOB'  => [ 'DBD::SQLite::strglob', 'DBD::SQLite::strglob' ],
+  'REGEXP'=> [ '=~',   '=~' ],
+  ) : ()),
+  (SQLITE_3021000 ? (
+  'NE'    => [ 'ne',   '!=' ],
+  'ISNOT' => [ 'defined',   'defined' ],
+  'ISNOTNULL' => [ 'defined',   'defined' ],
+  'ISNULL'    => [ '!defined',  '!defined' ],
+  'IS'    => [ '!defined',   '!defined' ],
+  ) : ()),
 );
 
 #----------------------------------------------------------------------
@@ -74,7 +88,7 @@ sub BEST_INDEX {
   # in FILTER() for deciding which rows match the constraints.
   my @conditions;
   my $ix = 0;
-  foreach my $constraint (grep {$_->{usable}} @$constraints) {
+  foreach my $constraint (grep {$_->{usable} and exists $SQLOP2PERLOP{ $_->{op} } } @$constraints) {
     my $col = $constraint->{col};
     my ($member, $optype);
 
@@ -95,12 +109,24 @@ sub BEST_INDEX {
       $optype  = $self->{optypes}[$col];
     }
     my $op = $SQLOP2PERLOP{$constraint->{op}}[$optype];
-    push @conditions,
-      "(defined($member) && defined(\$vals[$ix]) && $member $op \$vals[$ix])";
-      # Note : $vals[$ix] refers to an array of values passed to the
-      # FILTER method (see below); so the eval-ed perl code will be a
-      # closure on those values
-
+    if (SQLITE_3021000 && $op =~ /defined/) {
+      if ($constraint->{op} =~ /NULL/) {
+        push @conditions,
+          "($op($member))";
+      } else {
+        push @conditions,
+          "($op($member) && !defined(\$vals[$ix]))";
+      }
+    } elsif (SQLITE_3010000 && $op =~ /str/) {
+      push @conditions,
+        "(defined($member) && defined(\$vals[$ix]) && !$op(\$vals[$ix], $member))";
+    } else {
+      push @conditions,
+        "(defined($member) && defined(\$vals[$ix]) && $member $op \$vals[$ix])";
+    }
+    # Note : $vals[$ix] refers to an array of values passed to the
+    # FILTER method (see below); so the eval-ed perl code will be a
+    # closure on those values
     # info passed back to the SQLite core -- see vtab.html in sqlite doc
     $constraint->{argvIndex} = $ix++;
     $constraint->{omit}      = 1;
@@ -194,7 +220,7 @@ sub FILTER {
 
   # print STDERR "PERL CODE:\n", $perl_code, "\n";
 
-  $self->{is_wanted_row} = eval $perl_code
+  $self->{is_wanted_row} = do { no warnings; eval $perl_code }
     or die "couldn't eval q{$perl_code} : $@";
 
   # position the cursor to the first matching row (or to eof)
@@ -363,7 +389,7 @@ time.  Here is a way to do it with a virtual table :
   my @files = ... ; # list of files to inspect
 
   # apply the L<stat> function to each file
-  our $file_stats = [ map {($_, stat $_)} @files];
+  our $file_stats = [ map { [ $_, stat $_ ] } @files];
 
   # create a temporary virtual table
   $dbh->do(<<"");
